
/* 
   Project:     SOFTSUSY 1.2
   File:        softpars.h
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: Soft SUSY breaking parameters

   $Log: softpars.h,v $
   Revision 1.8  2002/07/30 12:57:32  allanach
   SOFTSUSY1.5

   Revision 1.7  2002/06/14 16:26:30  allanach
   Switches included for 2-loop running of scalar masses, and calulating mt at mt.

   Revision 1.6  2002/04/26 15:14:44  allanach
   Deleted all translation routines and defined boundary conditions within
   softsusy.h and softsusy.cpp

   Revision 1.5  2002/04/14 13:50:41  allanach
   Now use V=m3^2 H1 H2 instead of V=mu B H1 H2. It's more natural!

   Revision 1.4  2002/04/12 16:51:27  allanach
   Added display/set functions to work automatically

   Revision 1.3  2002/04/12 06:24:50  allanach
   Code maintenance - returning a subobject made simpler

   Revision 1.3  2001/09/28 14:49:02  allanach
   GMSB boundary conditions added

   Revision 1.2  2001/07/18 14:42:51  allanach
   Added proper header info
*/

#ifndef SOFTPARS_H
#define SOFTPARS_H

#include "susy.h"
#include "def.h"
#include "linalg.h"

typedef enum {mQl=1, mUr, mDr, mLl, mEr} softMasses;
typedef enum {UA=1, DA, EA} trilinears;

const static int numSoftParsMssm = 110;

class SoftParsMssm: public MssmSusy
{
private:
  DoubleVector mGaugino; // Gaugino masses, see ::beta for definitions
  DoubleMatrix ua, da, ea; // Trilinear soft terms..
  DoubleMatrix mQLsq, mURsq, mDRsq, mLLsq, mSEsq; // soft mass squared matrices
  double m3sq, mH1sq, mH2sq; // Bilinear Higgs soft parameters
public:
  SoftParsMssm();
  SoftParsMssm(const MssmSusy &);
  SoftParsMssm(const SoftParsMssm &);
  const SoftParsMssm & operator=(const SoftParsMssm & s);
  SoftParsMssm(const MssmSusy & s, const DoubleVector & mG, const
	       DoubleMatrix & aU, const DoubleMatrix & aD, const DoubleMatrix
	       & aE, const DoubleMatrix & mQl, const DoubleMatrix & mUr, const
	       DoubleMatrix & mDr, const DoubleMatrix & mLl, const
	       DoubleMatrix & mEr, double m3sq, double mH1sq, double mH2sq,
	       double mu, int l, int t);

  inline SoftParsMssm displaySoftPars() const;
  DoubleMatrix displayTrilinear(trilinears) const;
  double displaySoftA(trilinears, int, int) const;
  DoubleMatrix displaySoftMassSquared(softMasses) const;
  inline double displayM3Squared() const;
  inline double displayMh1Squared() const;
  inline double displayMh2Squared() const;
  inline DoubleVector displayGaugino() const;
  virtual DoubleVector display() const;

  void setSoftPars(SoftParsMssm const &);
  void setSoftMassElement(softMasses, int, int, double);
  void setSoftMassMatrix(softMasses, const DoubleMatrix &);
  void setTrilinearMatrix(trilinears, const DoubleMatrix &);
  void setTrilinearElement(trilinears k, int i, int j, double a);
  void setGauginoMass(int, double);
  void setAllGauginos(const DoubleVector &);
  void setM3Squared(double);
  void setMh1Squared(double);
  void setMh2Squared(double);
  void set(const DoubleVector &);
  void setSusy(const MssmSusy &);

  DoubleVector beta() const; 
  SoftParsMssm beta2() const; 
  void anomalousDeriv(DoubleMatrix & gEE, DoubleMatrix & gLL,
		      DoubleMatrix & gQQ, DoubleMatrix & gUU,
		      DoubleMatrix & gDD, 
		      double & gH1H1, double & gH2H2) const;
  void yTildes(DoubleMatrix & yu, DoubleMatrix & yd, DoubleMatrix &ye) const;
  
  void universal(double,  double,  double,  double, double);
  
  void addAmsb(double m32);
  void standardSugra(double, double, double);
  void universalScalars(double m0);
  void universalGauginos(double m12);
  void universalTrilinears(double a0);
  void minimalGmsb(int n5, double lambda, double mMess);  

  void inputSoftParsOnly();
};

ostream & operator <<(ostream &left, const SoftParsMssm &s);
istream & operator >>(istream &left, SoftParsMssm &s);

inline SoftParsMssm::SoftParsMssm()
  : MssmSusy(), mGaugino(3), ua(3, 3), da(3, 3), ea(3, 3), 
  mQLsq(3, 3), mURsq(3, 3), mDRsq(3, 3), mLLsq(3, 3), mSEsq(3, 3), m3sq(0.0),
  mH1sq(0.0), mH2sq(0.0) {      
  
  setPars(numSoftParsMssm);
  setMu(0.0);
  setLoops(0);
  setThresholds(0);
}

inline SoftParsMssm::SoftParsMssm(const SoftParsMssm & s)
  : MssmSusy(s.displaySusy()), 
  mGaugino(s.displayGaugino()), ua(s.displayTrilinear(UA)),
  da(s.displayTrilinear(DA)), ea(s.displayTrilinear(EA)),
  mQLsq(s.displaySoftMassSquared(mQl)), 
  mURsq(s.displaySoftMassSquared(mUr)), 
  mDRsq(s.displaySoftMassSquared(mDr)),
  mLLsq(s.displaySoftMassSquared(mLl)),
  mSEsq(s.displaySoftMassSquared(mEr)), 
  m3sq(s.displayM3Squared()), mH1sq(s.displayMh1Squared()),
  mH2sq(s.displayMh2Squared()) {
  
  setPars(numSoftParsMssm);
  setMu(s.displayMu()); 
  setLoops(s.displayLoops());
  setThresholds(s.displayThresholds());
}

inline SoftParsMssm::SoftParsMssm(const MssmSusy &s)
  : MssmSusy(s), mGaugino(3), ua(3, 3), da(3, 3), ea(3, 3), 
    mQLsq(3, 3), mURsq(3, 3), mDRsq(3, 3), mLLsq(3, 3), mSEsq(3, 3), m3sq(0.0),
    mH1sq(0.0),  mH2sq(0.0) { 
      setPars(numSoftParsMssm);
      setMu(s.displayMu()); 
      setLoops(s.displayLoops());
      setThresholds(s.displayThresholds());
}

inline SoftParsMssm::SoftParsMssm
(const MssmSusy & s, const DoubleVector & mG, const
 DoubleMatrix & aU, const DoubleMatrix & aD, const DoubleMatrix & aE, const
 DoubleMatrix & mQl, const DoubleMatrix & mUr, const DoubleMatrix & mDr, const
 DoubleMatrix & mLl, const DoubleMatrix & mEr, double m3sqn, double mH1sq,
 double mH2sq, double mu, int l, int t)
  : MssmSusy(s), mGaugino(mG), ua(aU), da(aD), ea(aE),
    mQLsq(mQl), mURsq(mUr), mDRsq(mDr), mLLsq(mLl), mSEsq(mEr), m3sq(m3sqn),
    mH1sq(mH1sq), mH2sq(mH2sq) {
      setPars(numSoftParsMssm);
      setMu(mu);
      setLoops(l);
      setThresholds(t);
}

inline SoftParsMssm SoftParsMssm::displaySoftPars() const { return *this; }

inline double SoftParsMssm::displayM3Squared() const { return m3sq; }

inline double SoftParsMssm::displayMh1Squared() const { return mH1sq; }

inline double SoftParsMssm::displayMh2Squared() const { return mH2sq; }

inline DoubleVector SoftParsMssm::displayGaugino() const { return mGaugino; }

inline void SoftParsMssm::setGauginoMass(int i, double f) { 
  mGaugino(i) = f; 
}

inline void SoftParsMssm::setM3Squared(double f) { m3sq = f; }
inline void SoftParsMssm::setMh1Squared(double f) { mH1sq = f; }
inline void SoftParsMssm::setMh2Squared(double f) { mH2sq = f; }
inline void SoftParsMssm::setSoftPars(SoftParsMssm const & s) { *this = s; }

#endif
