
/* 
   Project:     SOFTSUSY 
   File:        softpoint.cpp
   Author:      Ben Allanach 
   Manual:      hep-ph/0104145, Comp. Phys. Comm. 143 (2002) 305 
   Webpage:     http://allanach.home.cern.ch/allanach/softsusy.html
   Description: main calling program: command line interface

   $Log: softpoint.cpp,v $
   Revision 1.11  2002/09/09 10:42:54  allanach
   TOLERANCE replaces EPS as being more user-friendly

   Revision 1.10  2002/09/09 10:21:12  allanach
   For gauge unification, enter "?" for mgut

   Revision 1.9  2002/09/03 14:29:38  allanach
   EPS, PRINTOUT, MIXING now all set in massIn

   Revision 1.8  2002/09/03 14:16:44  allanach
   Taken PRINTOUT, MIXING, EPS out of def.h to make it quicker to compile once
   they are changed.

   Revision 1.7  2002/07/30 12:57:32  allanach
   SOFTSUSY1.5

   Revision 1.6  2002/07/19 15:54:07  allanach
   SOFTSUSY1.5 version

   Revision 1.4  2002/04/26 15:14:44  allanach
   Deleted all translation routines and defined boundary conditions within
   softsusy.h and softsusy.cpp

   Revision 1.3  2002/04/25 12:19:11  allanach
   Cleared bug that happened if there was no argument to the function

   Revision 1.1  2002/03/01 17:33:58  allanach
   Added softpoint: allows you to call softsusy just from the command line.
   Also, interfaces to micromegas added. All interfaces write to input files:
   SoftsusyMicromegasInterface, SoftsusyIsawigInterface,
   SoftsusyIsajetInterface, SoftsusySsrunInterface respectively.
*/

#include "iostream.h"
#include "complex.h"
#include "def.h"
#include "linalg.h"
#include "lowe.h"
#include "rge.h"
#include "softsusy.h"
#include "softpars.h"
#include "susy.h"
#include "utils.h"
#include "numerics.h"

int MIXING, PRINTOUT;
double TOLERANCE;

// Does the user require gauge unification or not?
double mgutCheck(char * a) {
  if (!strcmp(a, "?")) return -1.0e16;
  else return atof(a);
}

void errorCall() {
    cout << "SOFTSUSY called with incorrect arguments. Need to put either:\n";
    cout << "softpoint.x sugra <m0> <m12> <a0> <tanb> <mgut> <sgnMu>\n";
    cout << "softpoint.x amsb <m0> <m32> <tanb> <mgut> <sgnMu>\n";
    cout << "softpoint.x gmsb <n5> <mMess> <lambda> <tanb> <sgnMu> \n";
    cout << "softpoint.x general <tanb> <mgut> <sgnMu> < softBreakingInput\n";
    cout << "where bracketed entries are numerical values and ";
    cout << "softBreakingInput ";
    cout << "is a file containing a set of SUSY breaking parameters \n";
    cout << "(in SOFTSUSY format) to be applied at mgut. \nFor";
    cout << " GAUGE UNIFICATION, enter ? for <mgut>.\n";
    exit(-1);
}

int main(int argc, char *argv[]) {

  // Sets format of output: 2 decimal places
  outputCharacteristics(4);

  void (*boundaryCondition)(MssmSoftsusy &, const DoubleVector &);

  cout << "SOFTSUSY1.6 command-line interface, Ben Allanach 2002\n";
  cout << "If you use SOFTSUSY, please refer to B.C. Allanach, \n";
  cout << " Comput. Phys. Commun. 143 (2002) 305, hep-ph/0104145\n\n";

  double mgut, tanb;
  int sgnMu;

  // If there are no arguments, give error message,
  // or if none of the options are called, then go to error message
  if (argc == 1 || (strcmp(argv[1], "sugra") && strcmp(argv[1], "amsb") &&
		    strcmp(argv[1], "gmsb") && strcmp(argv[1], "general")))
    errorCall();

  DoubleVector pars(3); 
  if (!strcmp(argv[1], "sugra")) {
    cout << "SOFTSUSY SUGRA calculation" << endl;
    boundaryCondition = &sugraBcs;
    if (argc == 8) {
      double m0 = atof(argv[2]);
      double m12 = atof(argv[3]);
      double a0 = atof(argv[4]);
      tanb = atof(argv[5]);
      mgut = mgutCheck(argv[6]);
      sgnMu =  atoi(argv[7]);
      pars(1) = m0; pars(2) = m12; pars(3) = a0;
    }
    else
      errorCall();
    // end of SUGRA option
  }

  if (!strcmp(argv[1], "amsb")) {
    cout << "SOFTSUSY mAMSB calculation" << endl;
    boundaryCondition = &amsbBcs;
       if (argc == 7) {
      double m0 = atof(argv[2]);
      double m32 = atof(argv[3]);
      tanb = atof(argv[4]);
      mgut = mgutCheck(argv[5]);
      sgnMu =  atoi(argv[6]);
      pars(1) = m32; pars(2) = m0; 
    }
    else 
      errorCall();
  }

  if (!strcmp(argv[1], "gmsb")) {
    cout << "SOFTSUSY mGMSB calculation" << endl;
    
    boundaryCondition = &gmsbBcs;
    
    if (argc == 7) {
      double n5 = atof(argv[2]);
      double mMess = atof(argv[3]);
      double lambda = atof(argv[4]);
      tanb = atof(argv[5]);
      sgnMu =  atoi(argv[6]);
      mgut = mMess;
      pars(1) = n5; pars(2) = mMess; pars(3) = lambda;
      if (lambda > mMess) {
	cout << "Input lambda=" << lambda << " should be greater than mMess="
	     << mMess << endl;
	exit(-1);
      }
    }
    else 
      errorCall();
  }

  if (!strcmp(argv[1], "general")) {
    cout << "SOFTSUSY general calculation" << endl;
    
    boundaryCondition = &generalBcs;
    
    if (argc == 5) {
      tanb  = atof(argv[2]);
      mgut  = mgutCheck(argv[3]);
      sgnMu = atoi(argv[4]);
      SoftParsMssm in;
      in.inputSoftParsOnly();
      pars.setEnd(110); pars = in.display();
    }
    else 
      errorCall();
  }

  QedQcd oneset;

  readIn(oneset, "massIn");

  cout << "Low energy data in SOFTSUSY: MIXING=" << MIXING << " TOLERANCE=" 
       << TOLERANCE << endl << oneset << endl;

  oneset.toMz();

  MssmSoftsusy r;   
        
  r.lowOrg(boundaryCondition, mgut, pars, sgnMu, tanb, oneset);

  if (r.displayProblem().test()) {
    cout << "SOFTSUSY problem with point: " << r.displayProblem();
    exit(-1);
  }

  // Interfaces to Monte Carlo codes:
  r.microMegasInterface("SoftSusyMicromegasInterface");
  r.isawigInterface751
    ("softsusy.out", "softsusy.in", "SoftSusyIsawigInterface");
  r.isajetInterface751("SoftSusyIsajetInterface");
  r.ssrunInterface751("softsusy.out", "SoftSusySsrunInterface");

  cout << "mgut=" << mgut << endl << r;

  cout << "Fine-tuning " << r.fineTune(sugraBcs, pars, mgut);

  // Successful completion - returns 0 as the exiting value
  exit(0);
}
